''' This class simulates a Wallet of an investor, in which the inverstor 
has a certain amount of money in two possible currencies USD or BitCoint(BC).
It receives as input parameter the values and the moving mean, and also
considers the time index.'''

class Wallet:
	usd=0
	bc=0
	index=0
	commission = 1.5 # commission in USD of each change
	def __init__(self, warehouse, initialUsd):
		self.wh = warehouse
		self.usd=initialUsd
		
	# It returns the price of BC
	def priceBc(self):
		if self.index<len(self.wh.prices):
			index = self.index
		else:
			index = len(self.wh.prices)-1;
		return self.wh.prices[index]

	# It returns the movile mean
	def movMean(self):
		return self.wh.movMeans[self.index]

	# It returns the previous moving mean with a certain offset
	def prevMovMean(self, offset):
		return self.wh.movMeans[max(0, self.index - offset)]

	# It converts the potential value of all in USD
	def getTotalUsd(self):
		return self.usd+self.bc*self.priceBc();

	# Convert all to Bitcoin
	def toBc(self):
		if self.usd>self.commission:
			self.usd=self.usd-self.commission
			self.bc= self.bc + self.usd/self.priceBc()
			self.usd=0
	
	# Convert all to USD
	def toUsd(self):
		if self.bc > 0:
			self.usd = self.usd + self.bc * self.priceBc()
			self.bc=0
			self.usd = self.usd - self.commission

	# It returs a string indiciating the current money in each currency
	def toStr(self):
		return "USD: "+str(self.usd)+"    BC: "+str(self.bc)

	# It indicates if the wallet has a next element in the time series to go forward
	def hasNext(self):
		return self.index < len(self.wh.times)

	# It moves to the next position
	def next(self):
		self.index = self.index+1

	# It moves the index to zero
	def restart(self):
		self.index = 0;


