import pandas as pd
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
from datetime import datetime
import math

''' This class gets real historic data with some minor preprocessing.
For example, it gets the prices and moving mean '''

class Warehouse:
	filename="C:/Users/Iván García-Magariño/trab/NoSinc/bitcoin-historical-data/coinbaseUSD_1-min_data_2014-12-01_to_2019-01-09.csv"
	# filename= "data.csv"
	init=0 # number of minutes before the end for starting retrieving data
	def __init__(self):
		self.df = pd.read_csv(self.filename)

	# It returns the series of times, prices and moving means
	# init: number of minutes before the end in which 
	#  it starts the retrieved data
	# seriesSize: Number of elements of the series
	def retrieveSeries(self, init, seriesSize, winSizeMM):		
		if(self.init!=init or self.seriesSize!=seriesSize or self.winSizeMM!=winSizeMM):
			self.init = init
			self.seriesSize=seriesSize
			self.winSizeMM=winSizeMM
			self.times = np.array([])
			self.prices = np.array([])
			self.movMeans = np.array([])
			end = self.init - self.seriesSize
			sumMM = 0
			countMM = 0
			for i in range(max(0, len(self.df.values)-self.init), max(0, len(self.df.values)-end)):
				price = self.df.at[i, 'Open']
				if not math.isnan(price):
					self.times=np.append(self.times, datetime.fromtimestamp(self.df.at[i, 'Timestamp']))	
					self.prices = np.append(self.prices, price)
					sumMM = sumMM + price
					countMM = countMM + 1
					if countMM>=self.winSizeMM:
						firstValue = self.df.at[i-self.winSizeMM,'Open']
						if not math.isnan(firstValue):
							sumMM = sumMM - firstValue
							countMM = countMM -1 
					mm = sumMM/countMM
					self.movMeans = np.append(self.movMeans, mm)
		return self.times, self.prices, self.movMeans
	
	def plot(self):
		fig, ax = plt.subplots()
		ax.plot_date(self.times, self.prices, fmt="-", label='Bitcoin Value')
		ax.plot(self.times, self.movMeans, label="Moving Mean")
		ax.set(xlabel='Date (yyyy-mm-dd)', ylabel='Bitcoin Price (USD)')
		ax.legend()
		plt.show()


	'''

	# It returns the series of times, prices and moving means
	# init: number of minutes before the end in which 
	#  it starts the retrieved data
	# seriesSize: Number of elements of the series
	def retrieveSeries(self, init, seriesSize, winSizeMM):		
		if(self.init!=init or self.seriesSize!=seriesSize or self.winSizeMM!=winSizeMM):
			self.init = init
			self.seriesSize=seriesSize
			self.winSizeMM=winSizeMM
			end = self.init - self.seriesSize
			sumMM = 0
			countMM = 0
			beginRange= max(0, len(self.df.values)-self.init)
			endRange = max(0, len(self.df.values)-end)
			sizeRange = endRange-beginRange
			self.times =  np.array([]) # np.empty([sizeRange])
			self.prices = np.zeros(sizeRange) #np.array([])
			self.movMeans = np.zeros(sizeRange) #np.array([])
			for i in range(beginRange, endRange):
				price = self.df.at[i, 'Open']
				if not math.isnan(price):
					self.times=np.append(self.times, datetime.fromtimestamp(self.df.at[i, 'Timestamp']))	
					#self.times[i-beginRange] = datetime.fromtimestamp(self.df.at[i, 'Timestamp'])	
					#self.prices = np.append(self.prices, price)
					self.prices[i-beginRange] = price
					sumMM = sumMM + price
					countMM = countMM + 1
					if countMM>=self.winSizeMM:
						firstValue = self.df.at[i-self.winSizeMM,'Open']
						if not math.isnan(firstValue):
							sumMM = sumMM - firstValue
							countMM = countMM -1 
					mm = sumMM/countMM
					#self.movMeans = np.append(self.movMeans, mm)
					self.movMeans[i-beginRange] = mm
		return self.times, self.prices, self.movMeans
	'''