###### Main ######

from Wallet import Wallet
from Warehouse import Warehouse
from Investor import Investor, InvestorTrendMM
from Simulator import Simulator
import numpy as np
import random
import matplotlib
import matplotlib.pyplot as plt

seriesSize = 20000
winSizeMM = 1000
initialUsd = 3600



# It runs the experiments on one investor and a wallet 
# and add it to the the results
def expInvestor(investor, wallet, results):
	#results = np.array([])
  #for i in range(len(wallets)):
	investor.wallet = wallet
	sim = Simulator(investor)
	sim.run()
	return np.append(results, investor.wallet.getTotalUsd());



# It gets the series for several investors
def expCompareInvestors(numSims, seriesSize, initBegin=25000, printValues=True):
	#random.seed(30)
	resTrendMM = np.array([])
	resSimple = np.array([])
	invTrendMM = InvestorTrendMM()
	invSimple = Investor()
	if printValues:
		print("TrendMM \t Simple")
	for i in range(numSims):
		init = random.randint(initBegin, 1000000)
		wh = Warehouse();		
		wh.retrieveSeries(init, seriesSize,winSizeMM)
		wallet = Wallet(wh, initialUsd)
		resTrendMM = expInvestor(invTrendMM, wallet, resTrendMM)
		resSimple = expInvestor(invSimple, wallet, resSimple)
		if printValues:
			print(str(resTrendMM[i])+"\t"+str(resSimple[i]))
	return resTrendMM, resSimple;	
	

# It plots a comparison of investor
def plotCompareInvestors(resTrendMM, resSimple):
	sims = np.arange(len(resTrendMM)) 
	fig, ax = plt.subplots()
	ax.plot(sims, resTrendMM, label='Results Trend MM')
	ax.plot(sims, resSimple,  label='Results Simple')
	ax.set(xlabel='Simulation Number (#)', ylabel='Resulting capital (USD)')
	ax.legend()
	plt.show()


#### Figure Comparison 40 cases
def figCompFixedIntervalCases():
	numSims = 40
	seriesSize = 20000
	winSizeMM = 1000
	resTrendMM, resSimple = expCompareInvestors(numSims, seriesSize)
	plotCompareInvestors(resTrendMM, resSimple)


def plotSeriesSizes(sizes, resTrendMM, resSimple):
	fig, ax = plt.subplots()
	ax.plot(sizes, resTrendMM, label='Results Trend MM')
	ax.plot(sizes, resSimple,  label='Results Simple')
	ax.set(xlabel='Simulated Time (min)', ylabel='Resulting capital (USD)')
	ax.legend()
	plt.show()

#### Figure different sizes
def figSeriesSizes():
	numSims = 5
	winSizeMM = 1000
	maxSizes = 105000
	sizes = np.arange(5000, maxSizes, 5000)
	print("Sizes: "+str(sizes))
	resTs = np.array([])
	resSs = np.array([])
	print("Size \t TrendMM \t Simple")
	for seriesSize in sizes:
		resTrendMM, resSimple = expCompareInvestors(numSims, seriesSize, maxSizes, True)
		resT = np.average(resTrendMM)
		resS = np.average(resSimple)
		resTs = np.append(resTs, resT)
		resSs = np.append(resSs, resS)
		print(str(seriesSize)+"\t"+str(resT)+"\t"+str(resS))
	plotSeriesSizes(sizes, resTs, resSs)

	



############# MAIN ###################

# figCompFixedIntervalCases()
figSeriesSizes()

'''

	 investor = InvestorTrendMM(w, wh)

# Create Warehouse
wh = Warehouse();
wh.retrieveSeries(init, seriesSize,winSizeMM)
#wh.plot()
# Create Wallet
w = Wallet(wh, initialUsd)
#investor = Investor(w, wh)
investor = InvestorTrendMM(w, wh)
sim = Simulator(w, investor)
sim.run()
sim.plot()



# It runs the experiments on one investor and several wallets 
# with the corresponding time series of Bitcoin.
# wallets: an np array of wallets 
# returns the resulting sereies of capital in USD
def expInvestorSeveral(investor, wallets):
	results = np.array([])
	for i in range(len(wallets)):
		investor.wallet = wallets[i]
		sim = Simulator(investor)
		sim.run()
		results = np.append(results, investor.wallet.getTotalUsd())
	return results;



# It gets the series for several investors
def expCompareInvestors():
	random.seed(30)
	numSims = 4
	wallets = np.array([])
	for i in range(numSims):
		init = random.randint(25000, 1000000)
		print(init)
		wh = Warehouse();		
		wh.retrieveSeries(init, seriesSize,winSizeMM)
		wallet = Wallet(wh, initialUsd)
		wallets = np.append(wallets, wallet)
	resultsTrendMM = expInvestor(InvestorTrendMM(wallet), wallets)
	print("Results TrendMM: "+str(resultsTrendMM))
	resultsSimple = expInvestor(Investor(wallet), wallets)
	print("Results Simple: "+str(resultsSimple))
	plotCompareInvestors(resultsTrendMM, resultsSimple)


'''

